#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>

#include "vector.h"
#include "pixel.h"
#include "camera.h"

// create a new camera description
void camera_init(FILE *in,camera_t **cam,int attrmax)
{
  // malloc() a camera_t structure, use memset() to
  // initialize it to 0 and store the CAM_COOKIE
  *cam = (camera_t *)malloc(sizeof(camera_t));
  memset((void *)*cam,0,sizeof(camera_t));
  (*cam)->cookie = CAM_COOKIE;

  // load attributes
  camera_load_attributes(in,*cam);
}

// read in camera attributes
void camera_load_attributes(FILE *in, camera_t *cam)
{
	char c,attrname[NAME_LEN];
	int count;

  // assume file pointer has already consumed 'camera' token
  assert((count = fscanf(in,"%s",cam->name))==1);

  // consume all chars until we get to '{'
  while((c=fgetc(in)) != EOF && c != '{');

  // loop until we hit '}'
  while((c=fgetc(in)) != EOF && c != '}') {
    // put character back into input stream
    fputc(c,in);

    assert((count = fscanf(in,"%s",attrname))==1);

    if(!strcmp(attrname,"pixeldim")) {
      fscanf(in,"%d %d",&cam->pixel_dim[0],&cam->pixel_dim[1]);
    }
    if(!strcmp(attrname,"worlddim")) {
      fscanf(in,"%lf %lf",&cam->world_dim[0],&cam->world_dim[1]);
    }
    if(!strcmp(attrname,"viewpoint")) vec_read(in,cam->view_point);

    // consume whitespace
    while((c=fgetc(in)) != EOF && c != '\n');
  }
}

// print camera object
void camera_print(camera_t *cam, FILE *out)
{
  assert(cam->cookie == CAM_COOKIE);

  // print out 'camera' token and camera name
  fprintf(out,"camera %s\n",cam->name);
  fprintf(out,"{\n");

  // print out pixel dim
  fprintf(out,"   pixeldim %d %d\n",cam->pixel_dim[0],cam->pixel_dim[1]);

  // print out worlddim
  fprintf(out,"   worlddim %3.0lf %3.0lf\n",cam->world_dim[0],cam->world_dim[1]);
  // print out viewpoint
  vec_print(out,"   viewpoint",cam->view_point);

  // space out cameras by placing extra '\n' at end
  fprintf(out,"}\n\n");
}

char* camera_getname(camera_t *cam)
{
  return(cam->name);
}
