#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>

#include "vector.h"

// compute the vector sum v3 = v2 + v1
void vec_sum(vec_t v1,vec_t v2,vec_t v3)
{
	int	i;

  for(i=0;i<3;i++) v3[i] = v2[i] + v1[i];
}


// compute the vector difference  v3 = v2 - v1
void vec_diff(vec_t v1,vec_t v2,vec_t v3)
{
	int	i;

  for(i=0;i<3;i++) v3[i] = v2[i] - v1[i];
}

// compute the inner (dot) product:  prod = v1 dot v2
double vec_dot(vec_t v1,vec_t v2)
{
	int	i;
	double	s=0.0;

  for(i=0;i<3;i++) s += v1[i] * v2[i];

  return(s);
}

// compute the length of the vector v1
double vec_len(vec_t v1)
{
  return(sqrt(vec_dot(v1,v1)));
}

// computer v2 = s * v1 where s is a scalar
void vec_scale(double s,vec_t v1,vec_t v2)
{
	int	i;

  for(i=0;i<3;i++) v2[i] = s * v1[i];
}

// compute v2, a unit vector in the direction of v1, v2 = v1 / || v1 ||
void vec_unit(vec_t v1, vec_t v2)
{
  vec_scale(1.0/vec_len(v1),v1,v2);
}

// copy vector v1 to vector v2
void vec_copy(vec_t v1, vec_t v2)
{
  memcpy(v2,v1,sizeof(vec_t));
}

// compute the outer product of two input vectors
void vec_cross(vec_t v1, vec_t v2, vec_t v3)
{
	int	i;
	vec_t	w;

// w[0] = v1[1] * v2[2] - v1[2] * v2[1];
// w[1] = v1[2] * v2[0] - v1[0] * v2[2];
// w[2] = v1[0] * v2[1] - v1[1] * v2[0];

  for(i=0;i<3;i++)
    w[i] = v1[(3+i+1)%3] * v2[(3+i-1)%3] - v1[(3+i-1)%3] * v2[(3+i+1)%3];

   vec_copy(w,v3);
}

// project a vector onto a plane
void vec_project(vec_t n,vec_t v,vec_t u)
{
	vec_t	w;	// work vector

  // orthogonal projection u of vector v onto plane n is
  // u = v - (v.n)n

  vec_scale(-vec_dot(n,v),n,w);
  vec_sum(v,w,u);
  vec_unit(u,u);
}

// read values into vector v1 and return the number of values read
int vec_read(FILE  *in, vec_t v1)
{
	int	i;
	int	c=0;	// count

  for(i=0;i<3;i++) c += fscanf(in,"%lf",&v1[i]);

  return(c);
}

// print contents of vector using %8.3lf format, with \n after last element
void vec_print(FILE *out,char *label,vec_t vec)
{
	int	i;

  fprintf(out,"%s",label);
  for(i=0;i<3;i++)
    vec[i] < 0.0 ? fprintf(out," %4.1lf",vec[i]) : fprintf(out,"%2.0lf",vec[i]); 
  fprintf(out,"\n");
}

// return 1 if vector is non-zero
int vec_nonzero(vec_t vec)
{
	int	i;
	double	precision=0.0000001;

  for(i=0;i<3;i++) if(fabs(vec[i]) > precision) return(1);

  return(0);
}
